from django.db import models
import logging

from django.contrib.contenttypes.models import ContentType
try:  # new import added in Django 1.7
    from django.contrib.contenttypes.fields import GenericForeignKey
except ImportError:
    from django.contrib.contenttypes import generic
    GenericForeignKey = generic.GenericForeignKey


logger = logging.getLogger(__file__)


class URLChangeMethod(models.Model):
    content_type = models.ForeignKey(ContentType)
    object_id = models.TextField()
    content_object = GenericForeignKey('content_type', 'object_id')

    method_name = models.TextField()
    current_url = models.TextField(blank=True)
    old_urls = models.ManyToManyField(
        'OldURL',
        related_name='model_method'
    )

    class Meta:
        app_label = 'url_tracker'

    def __unicode__(self):
        return '{0}.{1}, with current url {2}'.format(
            self.content_object,
            self.method_name,
            self.current_url
        )


class OldURL(models.Model):
    url = models.TextField(unique=True)

    class Meta:
        app_label = 'url_tracker'

    def __unicode__(self):
        return '{0}'.format(self.url)

    def get_new_url(self):
        all_new_urls = self.model_method.order_by('-current_url').values_list('current_url', flat=True)
        new_url = all_new_urls[0]
        if len(all_new_urls) > 1:
            logger.warning(
                ('the url {0} has multiple new_urls associated with it'
                 '{1} was chosen out of {2}').format(
                     self,
                     new_url,
                     all_new_urls
                 )
            )
        return new_url



from django.contrib import admin

from url_tracker.models import URLChangeMethod, OldURL


class URLChangeMethodAdmin(admin.ModelAdmin):
    list_display = ('content_object', 'method_name', 'current_url')
    list_filter = ('content_type', 'method_name')


class OldURLAdmin(admin.ModelAdmin):
    list_display = ('url',)


admin.site.register(URLChangeMethod, URLChangeMethodAdmin)
admin.site.register(OldURL, OldURLAdmin)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='OldURL',
            fields=[
                ('id', models.AutoField(auto_created=True, serialize=False, verbose_name='ID', primary_key=True)),
                ('url', models.TextField(unique=True)),
            ],
        ),
        migrations.CreateModel(
            name='URLChangeMethod',
            fields=[
                ('id', models.AutoField(auto_created=True, serialize=False, verbose_name='ID', primary_key=True)),
                ('object_id', models.TextField()),
                ('method_name', models.TextField()),
                ('current_url', models.TextField(blank=True)),
                ('content_type', models.ForeignKey(to='contenttypes.ContentType')),
                ('old_urls', models.ManyToManyField(to='url_tracker.OldURL', related_name='model_method')),
            ],
        ),
    ]



import logging
import warnings

from django.db.models import signals
from django.core.exceptions import ImproperlyConfigured

logger = logging.getLogger(__file__)


def lookup_previous_url(instance, **kwargs):
    """
    Gets the previous urls for the model. It will save them too a
    URLChangeMethod object, as OldURLs. If the url method won't resolve
    then the url is treated as if it doesn't exist. Also it won't create
    a URLChangeMethod if the old_url doesn't exist or is equal to the current
    url of that model

    When this method is called `instance` represents the presaved version of the instance.
    So to get the old URL, we get the object from the database, to get its current state
    """
    for method_name in instance.get_url_tracking_methods():

        # If the instance is new then don't create a url change
        try:
            instance = instance.__class__.objects.get(pk=instance.pk)
        except instance.__class__.DoesNotExist:
            continue

        old_url = getattr(instance, method_name)()
        if not old_url:
            return
        add_old_url(instance, method_name, old_url)


def add_old_url(instance, method_name, old_url):
    '''
    Saves an old_url for a method and an instance.

    It will create a OldUrl object for that url and add it to the url_method for that instance.
    '''
    from url_tracker.models import URLChangeMethod, OldURL
    from django.contrib.contenttypes.models import ContentType

    content_type = ContentType.objects.get_for_model(instance.__class__)
    url_method, created = URLChangeMethod.objects.get_or_create(
        content_type=content_type,
        object_id=instance.pk,
        method_name=method_name
    )
    old_url, __ = OldURL.objects.get_or_create(url=old_url)
    url_method.old_urls.add(old_url)
    url_method.save()


def track_changed_url(instance, **kwargs):
    """
    Saves the current_url for an instance.

    If that instance has not URLChangeMethod, then it will not create one.
    """
    from url_tracker.models import URLChangeMethod
    from django.contrib.contenttypes.models import ContentType

    for method_name in instance.get_url_tracking_methods():
        content_type = ContentType.objects.get_for_model(instance.__class__)
        try:
            url_method = URLChangeMethod.objects.get(
                content_type=content_type,
                object_id=instance.pk,
                method_name=method_name
            )
        except URLChangeMethod.DoesNotExist:
            continue
        current_url = getattr(instance, method_name)()

        logger.debug(
            "tracking URL change for instance '%s' URL",
            instance.__class__.__name__
        )

        url_method.current_url = current_url
        url_method.old_urls.filter(url=current_url).delete()
        if not url_method.old_urls.exists():
            url_method.delete()
            continue
        url_method.save()


def track_url_changes_for_model(model, absolute_url_method='get_absolute_url'):
    """
    Register the *model* for URL tracking. It requires the *model* to provide
    an attribute ``url_tracking_methods`` and/or a ``get_url_tracking_methods``
    method to return a list of methods to retrieve trackable URLs.
    The default setup provides ``url_tracking_methods = ['get_absolute_url']``.

    The ``pre_save`` and ``post_save`` methods are connected
    to different tracking methods for *model* and create/update
    ``URLChangeRecord``s as required.
    """
    if not hasattr(model, 'get_url_tracking_methods'):
        from url_tracker.mixins import URLTrackingMixin

        warnings.warn(
            "the 'absolute_url_method' is deprecated, use the "
            "'UrlTrackingMixin' instead",
            PendingDeprecationWarning
        )
        model.url_tracking_methods = [absolute_url_method]
        model.get_url_tracking_methods = URLTrackingMixin.get_url_tracking_methods

    # make sure that URL method names are specified for the given model
    if not getattr(model, 'url_tracking_methods', None):
        raise ImproperlyConfigured("no URLs specified for model '%s'" % model)

    for method_name in model.get_url_tracking_methods():
        if not hasattr(model, method_name):
            raise ImproperlyConfigured(
                "model instance '%s' does not have a method '%s'" % (
                    model.__name__,
                    method_name
                )
            )

    signals.pre_save.connect(lookup_previous_url, sender=model, weak=False)
    signals.post_save.connect(track_changed_url, sender=model, weak=False)



#!/usr/bin/env python

from setuptools import setup, find_packages

setup(
    name="django-url-tracker-fork",
    version='0.2.0',
    url="https://github.com/saulshanabrook/django-url-tracker",
    author="Sebastian Vetter",
    author_email="sebastian.vetter@tangentone.com.au",
    description=("A little app that trackes URL changes in a database table "
                 "to provide HTTP 301 & 410 on request."),
    long_description = open('README.rst').read(),
    license = "BSD",
    packages = find_packages(exclude=["docs*", "tests*"]),
    include_package_data = True,
    install_requires=[
        'django>=1.3.1,<1.10',
    ],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "Programming Language :: Python",
        "License :: OSI Approved :: Apache Software License",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Topic :: Software Development"
    ],
    keywords = "seo, django, framework",
)



from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django import http


class URLChangePermanentRedirectMiddleware(object):
    def __init__(self):
        if 'url_tracker' not in settings.INSTALLED_APPS:
            raise ImproperlyConfigured(
                "You cannot use URLChangePermanentRedirectMiddleware when "
                "url_tracker is not installed."
            )

    def process_request(self, request):
        from url_tracker.models import OldURL

        full_path = request.get_full_path()

        old_url = None
        try:
            old_url = OldURL.objects.get(url__exact=full_path)
        except OldURL.DoesNotExist:
            pass
        if settings.APPEND_SLASH and not request.path.endswith('/'):
            # Try appending a trailing slash.
            path_len = len(request.path)
            full_path = full_path[:path_len] + '/' + full_path[path_len:]
            try:
                old_url = OldURL.objects.get(url__exact=full_path)
            except OldURL.DoesNotExist:
                pass
        if old_url:
            new_url = old_url.get_new_url()
            if not new_url:
                return http.HttpResponseGone()
            return http.HttpResponsePermanentRedirect(new_url)
